%Monte Carlo simulations (for DGP2) for the paper "Comparison of Methods for Constructing
%Joint Confidence Bands for Impulse Response Functions" by
%Helmut Luetkepohl, Anna Staszewska-Bystrova and Peter Winker, 2013.

clear
clc

randn('state',0);
rand('state',0);

mc=1000;  %number of Monte Carlo replications
n=5000;   %number of bootstrap replications
h=18;     %horizon (shock occurs at h=0)
sl=0.1;   %1-nominal coverage rate
tt=100;   %net sample size

lag_e=1;   %1 to estimate lag length in each MC replication, 0 to use true lag length
infoc=2;   %information criterion used to estimate number of lags: 1 for AICc, 2 for AIC, 3 for BIC
maxlag=12; % max. lag order

%**************************************************************************
% Load series
dprod= load (['dprod.asc']);
rea= load (['rea.asc']);
rpo= load (['rpo.asc']);

mx=[dprod rea rpo];
nvar=3;       %number of variables
nlags1=3;     %number of lags in the VAR
ct=tt+nlags1; %gross sample size

%estimation
[bm, um, sigmam, ym, xm] = olsvar(mx, nlags1);

b=bm;
sigma=sigmam;

%Response functions using true parameter values
[irf_true]=ortirf(b,nlags1,nvar,h,sigma);

traj=zeros(nvar^2,h+1);
for i=1:h+1
    for j=1:nvar
        traj(nvar*(j-1)+1:nvar*j,i)=irf_true(nvar*i-nvar+1:nvar*i,j);
    end
end

%**************************************************************************
naive_cov=zeros(mc,nvar^2);             %coverage of the naive band
naive_width=zeros(mc,nvar^2);           %width of the naive band

scheffe_cov=zeros(mc,nvar^2);           %coverage of the Scheffe band
scheffe_width=zeros(mc,nvar^2);         %width of the Scheffe band

scheffe_mod_cov=zeros(mc,nvar^2);       %coverage of the modified Scheffe band
scheffe_mod_width=zeros(mc,nvar^2);     %width of the modified Scheffe band

bonferroni_cov=zeros(mc,nvar^2);        %coverage of the Bonferroni (B) band
bonferroni_width=zeros(mc,nvar^2);      %width of the Bonferroni (B) band

bonferroni_adj_cov=zeros(mc,nvar^2);    %coverage of the adjusted Bonferroni (B-adj) band
bonferroni_adj_width=zeros(mc,nvar^2);  %width of the adjusted Bonferroni (B-adj) band

npaths_cov=zeros(mc,nvar^2);            %coverage of the NP band
npaths_width=zeros(mc,nvar^2);          %width of the NP band

WW_cov=zeros(mc,nvar^2);                %coverage of the WW band
WW_width=zeros(mc,nvar^2);              %width of the WW band
%**************************************************************************
tic

for m=1:mc                              %MC replications
    m

    %obtain a sample of data from the DGP
    [mx]=simdata(sigma,b,nlags1,nvar,ct);

    if lag_e==1
        nlags=lagest(mx,nvar,maxlag,infoc);
    else
        nlags=nlags1;
    end

    [bm, um, sigmam, ym, xm] = olsvar(mx, nlags); %estimate parameters of the model
    G0m=inv(xm'*xm)*(ct-nlags);

    %Scheffe's and the modified Scheffe's bands
    [sch,sch_mod]=asym_bands(h,nvar,ct,nlags,sl,bm,sigmam,G0m);

    %bootstrap replicates of the response functions, responses based on
    %bias-corrected estimators, stnadrdized prediction errors, estimated
    %variance matrix of the response coefficient estimators
    [bp1,resp,sterr,varm]=bpaths1(mx,nlags,nvar,ct,h,n);

    %NP band
    irfs=zeros(nvar,h+1);
    NP=zeros(2*nvar^2,h+1);
    k=0;
    for c=1:nvar
        for cc=1:nvar
            k=k+1;
            bb=bp1(n*(k-1)+1:k*n,:);
            irfs(cc,:)=resp((c-1)*nvar+cc,:);
            NP(2*k-1:2*k,:)=np(bb, irfs, (h+1), cc, sl, n);
        end
    end

    %WW band
    k=0;
    WW=zeros(2*nvar^2,h+1);
    for c=1:nvar
        for cc=1:nvar
            k=k+1;
            sterr1=sterr(n*(k-1)+1:k*n,:);
            WW(2*k-1:2*k,:)=ww(sterr1,resp(k,:),h+1,sl,n,varm,c,cc,nvar,ct,nlags);
        end
    end

    %naive band, Bonferroni's band (B) and adjusted Bonferroni's band (B-adj)
    N=zeros(2*nvar^2,h+1);
    B=zeros(2*nvar^2,h+1);
    Badj=zeros(2*nvar^2,h+1);
    B1=zeros(2,h+1);

    for c=1:nvar^2
        bb1=bp1(n*(c-1)+1:c*n,:);
        if bb1(:,1)==zeros(n,1)
            B1(1,:)=quantile(bb1,sl/(2*(h)));
            B1(2,:)=quantile(bb1,1-sl/(2*(h)));
        else
            B1(1,:)=quantile(bb1,sl/(2*(h+1)));
            B1(2,:)=quantile(bb1,1-sl/(2*(h+1)));
        end
        N(2*c-1,:)=quantile(bb1,sl/2);
        N(2*c,:)=quantile(bb1,1-sl/2);
        B(2*c-1:2*c,:)=B1;
        [b1,b2]=adjB(h+1,B(2*c-1,:),B(2*c,:),bb1,n,sl);
        Badj(2*c-1,:)=b1;
        Badj(2*c,:)=b2;
    end

    % check coverage and calculate width
    for z=1:nvar^2
        naive_cov(m,z)=inside(1,traj(z,:),N(2*z-1,:),N(2*z,:));
        scheffe_cov(m,z)=inside(1,traj(z,:),sch(2*z-1,:),sch(2*z,:));
        scheffe_mod_cov(m,z)=inside(1,traj(z,:),sch_mod(2*z-1,:),sch_mod(2*z,:));
        bonferroni_cov(m,z)=inside(1,traj(z,:),B(2*z-1,:),B(2*z,:));
        bonferroni_adj_cov(m,z)=inside(1,traj(z,:),Badj(2*z-1,:),Badj(2*z,:));
        npaths_cov(m,z)=inside(1,traj(z,:),NP(2*z-1,:),NP(2*z,:));
        WW_cov(m,z)=inside(1,traj(z,:),WW(2*z-1,:),WW(2*z,:));

        naive_width(m,z)=sos(N(2*z-1,:),N(2*z,:));
        scheffe_width(m,z)=sos(sch(2*z-1,:),sch(2*z,:));
        scheffe_mod_width(m,z)=sos(sch_mod(2*z-1,:),sch_mod(2*z,:));
        bonferroni_width(m,z)=sos(B(2*z-1,:),B(2*z,:));
        bonferroni_adj_width(m,z)=sos(Badj(2*z-1,:),Badj(2*z,:));
        npaths_width(m,z)=sos(NP(2*z-1,:),NP(2*z,:));
        WW_width(m,z)=sos(WW(2*z-1,:),WW(2*z,:));
    end
end

%prepare file with results "Summary" saved in the "Results" folder
out=output(mc,n,tt,sl,h,lag_e,naive_cov,scheffe_cov,scheffe_mod_cov,bonferroni_cov,bonferroni_adj_cov,npaths_cov,...
    WW_cov,naive_width,scheffe_width,scheffe_mod_width,bonferroni_width,bonferroni_adj_width,npaths_width,WW_width);

toc