%Empirical example for the paper "Comparison of Methods for Constructing
%Joint Confidence Bands for Impulse Response Functions" by 
%Helmut Luetkepohl, Anna Staszewska-Bystrova and Peter Winker, 2013.

clear
clc

n=5000;   %number of bootstrap replications
h=18;     %horizon (shock occurs at h=0)
sl=0.1;   %1-nominal coverage rate
nlags=3;  %number of lags in the VAR
%**************************************************************************
tic
% Load series
dprod= load (['dprod.asc']);
rea= load (['rea.asc']);
rpo= load (['rpo.asc']);

mx=[dprod rea rpo];
[ct,nvar]=size(mx);

%estimation
[bm, um, sigmam, ym, xm] = olsvar(mx, nlags);

%bootstrap replicates of the response functions, responses based on
%bias-corrected estimators, stnadrdized prediction errors, estimated
%variance matrix of the response coefficient estimators
[bp1,resp,sterr,varm]=bpaths1(mx,nlags,nvar,ct,h,n);

%NP band
irfs=zeros(nvar,h+1);
NP=zeros(2*nvar^2,h+1);
k=0;
for c=1:nvar
    for cc=1:nvar
        k=k+1;
        bb=bp1(n*(k-1)+1:k*n,:);
        irfs(cc,:)=resp((c-1)*nvar+cc,:);
        NP(2*k-1:2*k,:)=np(bb, irfs, (h+1), cc, sl, n);
    end
end

%WW band
k=0;
WW=zeros(2*nvar^2,h+1);
for c=1:nvar
    for cc=1:nvar
        k=k+1;
        sterr1=sterr(n*(k-1)+1:k*n,:);
        WW(2*k-1:2*k,:)=ww(sterr1,resp(k,:),h+1,sl,n,varm,c,cc,nvar,ct,nlags);
    end
end

%Bonferroni's band (B) and adjusted Bonferroni's band (B-adj)
B=zeros(2*nvar^2,h+1);
Badj=zeros(2*nvar^2,h+1);
B1=zeros(2,h+1);

for c=1:nvar^2
    bb1=bp1(n*(c-1)+1:c*n,:);
    if bb1(:,1)==zeros(n,1)
        B1(1,:)=quantile(bb1,sl/(2*(h)));
        B1(2,:)=quantile(bb1,1-sl/(2*(h)));
    else
        B1(1,:)=quantile(bb1,sl/(2*(h+1)));
        B1(2,:)=quantile(bb1,1-sl/(2*(h+1)));
    end
    B(2*c-1:2*c,:)=B1;
    [b1,b2]=adjB(h+1,B(2*c-1,:),B(2*c,:),bb1,n,sl);
    Badj(2*c-1,:)=b1;
    Badj(2*c,:)=b2;
end

%figure
r=[0;ones(h,1)];
rr=cumsum(r);
figure;
subplot(3,3,1)
plot(rr, Badj(1,:),'LineWidth',2, 'Color','blue');
hold on
plot(rr, Badj(2,:),'LineWidth',2, 'Color','blue');
hold on
plot(rr, B(1,:),'.', 'Color', 'black');
hold on
plot(rr, B(2,:),'.', 'Color', 'black');
hold on
plot(rr, WW(1,:),'-.', 'LineWidth',2, 'Color','r');
hold on
plot(rr, WW(2,:),'-.', 'LineWidth',2, 'Color','r');
hold on
plot(rr, NP(1,:),'--', 'LineWidth',2,  'Color','m');
hold on
plot(rr, NP(2,:),'--', 'LineWidth',2,  'Color','m');
hold on
plot(rr, resp(1,:), 'LineWidth',1,'Color','black');
xlim([0 h+1]);
hold off
subplot(3,3,2)
plot(rr, Badj(3,:),'LineWidth',2, 'Color','blue');
hold on
plot(rr, Badj(4,:),'LineWidth',2, 'Color','blue');
hold on
plot(rr, B(3,:),'.','Color', 'black');
hold on
plot(rr, B(4,:),'.','Color', 'black');
hold on
plot(rr, WW(3,:),'-.', 'LineWidth',2, 'Color','r');
hold on
plot(rr, WW(4,:),'-.', 'LineWidth',2, 'Color','r');
hold on
plot(rr, NP(3,:),'--', 'LineWidth',2,  'Color','m');
hold on
plot(rr, NP(4,:),'--', 'LineWidth',2,  'Color','m');
hold on
plot(rr, resp(2,:), 'LineWidth',1,'Color','black');
xlim([0 h+1]);
hold off
subplot(3,3,3)
plot(rr, Badj(5,:),'LineWidth',2, 'Color','blue');
hold on
plot(rr, Badj(6,:),'LineWidth',2, 'Color','blue');
hold on
plot(rr, B(5,:),'.','Color', 'black');
hold on
plot(rr, B(6,:),'.','Color', 'black');
hold on
plot(rr, WW(5,:),'-.', 'LineWidth',2, 'Color','r');
hold on
plot(rr, WW(6,:),'-.', 'LineWidth',2, 'Color','r');
hold on
plot(rr, NP(5,:),'--', 'LineWidth',2,  'Color','m');
hold on
plot(rr, NP(6,:),'--', 'LineWidth',2,  'Color','m');
hold on
plot(rr, resp(3,:), 'LineWidth',1,'Color','black');
xlim([0 h+1]);
hold off
subplot(3,3,4)
plot(rr, Badj(7,:),'LineWidth',2, 'Color','blue');
hold on
plot(rr, Badj(8,:),'LineWidth',2, 'Color','blue');
hold on
plot(rr, B(7,:),'.','Color', 'black');
hold on
plot(rr, B(8,:),'.','Color', 'black');
hold on
plot(rr, WW(7,:),'-.', 'LineWidth',2, 'Color','r');
hold on
plot(rr, WW(8,:),'-.', 'LineWidth',2, 'Color','r');
hold on
plot(rr, NP(7,:),'--', 'LineWidth',2,  'Color','m');
hold on
plot(rr, NP(8,:),'--', 'LineWidth',2,  'Color','m');
hold on
plot(rr, resp(4,:), 'LineWidth',1,'Color','black');
xlim([0 h+1]);
hold off
subplot(3,3,5)
plot(rr, Badj(9,:),'LineWidth',2, 'Color','blue');
hold on
plot(rr, Badj(10,:),'LineWidth',2, 'Color','blue');
hold on
plot(rr, B(9,:),'.','Color', 'black');
hold on
plot(rr, B(10,:),'.','Color', 'black');
hold on
plot(rr, WW(9,:),'-.', 'LineWidth',2, 'Color','r');
hold on
plot(rr, WW(10,:),'-.',  'LineWidth',2,'Color','r');
hold on
plot(rr, NP(9,:),'--', 'LineWidth',2,  'Color','m');
hold on
plot(rr, NP(10,:),'--', 'LineWidth',2,  'Color','m');
hold on
plot(rr, resp(5,:), 'LineWidth',1,'Color','black');
xlim([0 h+1]);
hold off
subplot(3,3,6)
plot(rr, Badj(11,:),'LineWidth',2, 'Color','blue');
hold on
plot(rr, Badj(12,:),'LineWidth',2, 'Color','blue');
hold on
plot(rr, B(11,:),'.','Color', 'black');
hold on
plot(rr, B(12,:),'.','Color', 'black');
hold on
plot(rr, WW(11,:),'-.', 'LineWidth',2, 'Color','r');
hold on
plot(rr, WW(12,:),'-.', 'LineWidth',2, 'Color','r');
hold on
plot(rr, NP(11,:),'--', 'LineWidth',2,  'Color','m');
hold on
plot(rr, NP(12,:),'--', 'LineWidth',2,  'Color','m');
hold on
plot(rr, resp(6,:), 'LineWidth',1,'Color','black');
xlim([0 h+1]);
hold off
subplot(3,3,7)
plot(rr, Badj(13,:),'LineWidth',2, 'Color','blue');
hold on
plot(rr, Badj(14,:),'LineWidth',2, 'Color','blue');
hold on
plot(rr, B(13,:),'.','Color', 'black');
hold on
plot(rr, B(14,:),'.','Color', 'black');
hold on
plot(rr, WW(13,:),'-.', 'LineWidth',2, 'Color','r');
hold on
plot(rr, WW(14,:),'-.', 'LineWidth',2, 'Color','r');
hold on
plot(rr, NP(13,:),'--', 'LineWidth',2,  'Color','m');
hold on
plot(rr, NP(14,:),'--', 'LineWidth',2,  'Color','m');
hold on
plot(rr, resp(7,:), 'LineWidth',1,'Color','black');
xlim([0 h+1]);
hold off
subplot(3,3,8)
plot(rr, Badj(15,:),'LineWidth',2, 'Color','blue');
hold on
plot(rr, Badj(16,:),'LineWidth',2, 'Color','blue');
hold on
plot(rr, B(15,:),'.','Color', 'black');
hold on
plot(rr, B(16,:),'.','Color', 'black');
hold on
plot(rr, WW(15,:),'-.', 'LineWidth',2, 'Color','r');
hold on
plot(rr, WW(16,:),'-.', 'LineWidth',2, 'Color','r');
hold on
plot(rr, NP(15,:),'--', 'LineWidth',2, 'Color','m');
hold on
plot(rr, NP(16,:),'--', 'LineWidth',2,  'Color','m');
hold on
plot(rr, resp(8,:), 'LineWidth',1,'Color','black');
xlim([0 h+1]);
hold off
subplot(3,3,9)
p1=plot(rr, Badj(17,:),'LineWidth',2, 'Color','blue');
hold on
p2=plot(rr, Badj(18,:),'LineWidth',2, 'Color','blue');
hold on
p3=plot(rr, B(17,:),'.', 'Color', 'black');
hold on
p4=plot(rr, B(18,:),'.', 'Color', 'black');
hold on
p5=plot(rr, WW(17,:),'-.', 'LineWidth',2, 'Color','r');
hold on
p6=plot(rr, WW(18,:),'-.', 'LineWidth',2, 'Color','r');
hold on
p7=plot(rr, NP(17,:),'--', 'LineWidth',2, 'Color','m');
hold on
p8=plot(rr, NP(18,:),'--','LineWidth',2,  'Color','m');
hold on
p11=plot(rr, resp(9,:), 'LineWidth',1,'Color','black');
xlim([0 h+1]);

hold off
h1=legend([p1 p3 p5 p7 p11],{'B-adj','B','WW', 'NP','response'},'Orientation','horizontal');
p = get(h1,'Position');
p(1) = 0;
p(2) = 0;
set(h1,'Position',p);
toc